@@ -29,31 +29,27 @@ exports.INSPECT_MAX_BYTES = 50;
 SlowBuffer.prototype.__proto__ = Buffer.prototype;
 
 
+function clamp(index, len, defaultValue) {
+  if (typeof index !== 'number') return defaultValue;
+  index = ~~index;  // Coerce to integer.
+  if (index >= len) return len;
+  if (index >= 0) return index;
+  index += len;
+  if (index >= 0) return index;
+  return 0;
+}
+
+
 function toHex(n) {
   if (n < 16) return '0' + n.toString(16);
   return n.toString(16);
 }
 
 
-SlowBuffer.prototype.hexSlice = function(start, end) {
-  var len = this.length;
-
-  if (!start || start < 0) start = 0;
-  if (!end || end < 0 || end > len) end = len;
-
-  var out = '';
-  for (var i = start; i < end; i++) {
-    out += toHex(this[i]);
-  }
-  return out;
-};
-
-
-
 SlowBuffer.prototype.toString = function(encoding, start, end) {
   encoding = String(encoding || 'utf8').toLowerCase();
   start = +start || 0;
-  if (typeof end == 'undefined') end = this.length;
+  if (typeof end !== 'number') end = this.length;
 
   // Fastpath empty strings
   if (+end == start) {
@@ -84,41 +80,11 @@ SlowBuffer.prototype.toString = function(encoding, start, end) {
       return this.ucs2Slice(start, end);
 
     default:
-      throw new Error('Unknown encoding');
+      throw new TypeError('Unknown encoding: ' + encoding);
   }
 };
 
 
-SlowBuffer.prototype.hexWrite = function(string, offset, length) {
-  offset = +offset || 0;
-  var remaining = this.length - offset;
-  if (!length) {
-    length = remaining;
-  } else {
-    length = +length;
-    if (length > remaining) {
-      length = remaining;
-    }
-  }
-
-  // must be an even number of digits
-  var strLen = string.length;
-  if (strLen % 2) {
-    throw new Error('Invalid hex string');
-  }
-  if (length > strLen / 2) {
-    length = strLen / 2;
-  }
-  for (var i = 0; i < length; i++) {
-    var byte = parseInt(string.substr(i * 2, 2), 16);
-    if (isNaN(byte)) throw new Error('Invalid hex string');
-    this[offset + i] = byte;
-  }
-  SlowBuffer._charsWritten = i * 2;
-  return i;
-};
-
-
 SlowBuffer.prototype.write = function(string, offset, length, encoding) {
   // Support both (string, offset, length, encoding)
   // and the legacy (string, encoding, offset, length)
@@ -170,35 +136,20 @@ SlowBuffer.prototype.write = function(string, offset, length, encoding) {
       return this.ucs2Write(string, offset, length);
 
     default:
-      throw new Error('Unknown encoding');
+      throw new TypeError('Unknown encoding: ' + encoding);
   }
 };
 
 
 // slice(start, end)
 SlowBuffer.prototype.slice = function(start, end) {
-  if (end === undefined) end = this.length;
-
-  if (end > this.length) {
-    throw new Error('oob');
-  }
-  if (start > end) {
-    throw new Error('oob');
-  }
-
-  return new Buffer(this, end - start, +start);
+  var len = this.length;
+  start = clamp(start, len, 0);
+  end = clamp(end, len, len);
+  return new Buffer(this, end - start, start);
 };
 
 
-function coerce(length) {
-  // Coerce length to a number (possibly NaN), round up
-  // in case it's fractional (e.g. 123.456). Since NaN
-  // comparisons are always false, use to return zero.
-  length = Math.ceil(+length);
-  return length > 0 ? length : 0;
-}
-
-
 var zeroBuffer = new SlowBuffer(0);
 
 // Buffer
@@ -212,30 +163,30 @@ function Buffer(subject, encoding, offset) {
   // Are we slicing?
   if (typeof offset === 'number') {
     if (!Buffer.isBuffer(subject)) {
-      throw new Error('First argument must be a Buffer when slicing');
+      throw new TypeError('First argument must be a Buffer when slicing');
     }
 
-    this.length = coerce(encoding);
+    this.length = +encoding > 0 ? Math.ceil(encoding) : 0;
     this.parent = subject.parent ? subject.parent : subject;
     this.offset = offset;
   } else {
     // Find the length
     switch (type = typeof subject) {
       case 'number':
-        this.length = coerce(subject);
+        this.length = +subject > 0 ? Math.ceil(subject) : 0;
         break;
 
       case 'string':
         this.length = Buffer.byteLength(subject, encoding);
         break;
 
-      case 'object': // Assume object is an array
-        this.length = coerce(subject.length);
+      case 'object': // Assume object is array-ish
+        this.length = +subject.length > 0 ? Math.ceil(subject.length) : 0;
         break;
 
       default:
-        throw new Error('First argument needs to be a number, ' +
-                        'array or string.');
+        throw new TypeError('First argument needs to be a number, ' +
+                            'array or string.');
     }
 
     if (this.length > Buffer.poolSize) {
@@ -257,14 +208,24 @@ function Buffer(subject, encoding, offset) {
       this.offset = 0;
     }
 
-    // Treat array-ish objects as a byte array.
-    if (isArrayIsh(subject)) {
-      for (var i = 0; i < this.length; i++) {
-        this.parent[i + this.offset] = subject[i];
+    // optimize by branching logic for new allocations
+    if (typeof subject !== 'number') {
+      if (type === 'string') {
+        // We are a string
+        this.length = this.write(subject, 0, encoding);
+      // if subject is buffer then use built-in copy method
+      } else if (Buffer.isBuffer(subject)) {
+        if (subject.parent)
+          subject.parent.copy(this.parent,
+                              this.offset,
+                              subject.offset,
+                              this.length + subject.offset);
+        else
+          subject.copy(this.parent, this.offset, 0, this.length);
+      } else if (isArrayIsh(subject)) {
+        for (var i = 0; i < this.length; i++)
+          this.parent[i + this.offset] = subject[i];
       }
-    } else if (type == 'string') {
-      // We are a string
-      this.length = this.write(subject, 0, encoding);
     }
   }
 
@@ -272,7 +233,7 @@ function Buffer(subject, encoding, offset) {
 }
 
 function isArrayIsh(subject) {
-  return Array.isArray(subject) || Buffer.isBuffer(subject) ||
+  return Array.isArray(subject) ||
          subject && typeof subject === 'object' &&
          typeof subject.length === 'number';
 }
@@ -280,6 +241,29 @@ function isArrayIsh(subject) {
 exports.SlowBuffer = SlowBuffer;
 exports.Buffer = Buffer;
 
+
+Buffer.isEncoding = function(encoding) {
+  switch (encoding && encoding.toLowerCase()) {
+    case 'hex':
+    case 'utf8':
+    case 'utf-8':
+    case 'ascii':
+    case 'binary':
+    case 'base64':
+    case 'ucs2':
+    case 'ucs-2':
+    case 'utf16le':
+    case 'utf-16le':
+    case 'raw':
+      return true;
+
+    default:
+      return false;
+  }
+};
+
+
+
 Buffer.poolSize = 8 * 1024;
 var pool;
 
@@ -313,15 +297,17 @@ Buffer.prototype.inspect = function inspect() {
 };
 
 
-Buffer.prototype.get = function get(i) {
-  if (i < 0 || i >= this.length) throw new Error('oob');
-  return this.parent[this.offset + i];
+Buffer.prototype.get = function get(offset) {
+  if (offset < 0 || offset >= this.length)
+    throw new RangeError('offset is out of bounds');
+  return this.parent[this.offset + offset];
 };
 
 
-Buffer.prototype.set = function set(i, v) {
-  if (i < 0 || i >= this.length) throw new Error('oob');
-  return this.parent[this.offset + i] = v;
+Buffer.prototype.set = function set(offset, v) {
+  if (offset < 0 || offset >= this.length)
+    throw new RangeError('offset is out of bounds');
+  return this.parent[this.offset + offset] = v;
 };
 
 
@@ -353,6 +339,9 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
   }
   encoding = String(encoding || 'utf8').toLowerCase();
 
+  if (string.length > 0 && (length < 0 || offset < 0))
+    throw new RangeError('attempt to write beyond buffer bounds');
+
   var ret;
   switch (encoding) {
     case 'hex':
@@ -385,7 +374,7 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
       break;
 
     default:
-      throw new Error('Unknown encoding');
+      throw new TypeError('Unknown encoding: ' + encoding);
   }
 
   Buffer._charsWritten = SlowBuffer._charsWritten;
@@ -394,17 +383,22 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
 };
 
 
+Buffer.prototype.toJSON = function() {
+  return Array.prototype.slice.call(this, 0);
+};
+
+
 // toString(encoding, start=0, end=buffer.length)
 Buffer.prototype.toString = function(encoding, start, end) {
   encoding = String(encoding || 'utf8').toLowerCase();
 
-  if (typeof start == 'undefined' || start < 0) {
+  if (typeof start !== 'number' || start < 0) {
     start = 0;
   } else if (start > this.length) {
     start = this.length;
   }
 
-  if (typeof end == 'undefined' || end > this.length) {
+  if (typeof end !== 'number' || end > this.length) {
     end = this.length;
   } else if (end < 0) {
     end = 0;
@@ -437,7 +431,7 @@ Buffer.prototype.toString = function(encoding, start, end) {
       return this.parent.ucs2Slice(start, end);
 
     default:
-      throw new Error('Unknown encoding');
+      throw new TypeError('Unknown encoding: ' + encoding);
   }
 };
 
@@ -455,22 +449,22 @@ Buffer.prototype.fill = function fill(value, start, end) {
   if (typeof value === 'string') {
     value = value.charCodeAt(0);
   }
-  if (!(typeof value === 'number') || isNaN(value)) {
-    throw new Error('value is not a number');
+  if (typeof value !== 'number' || isNaN(value)) {
+    throw new TypeError('value is not a number');
   }
 
-  if (end < start) throw new Error('end < start');
+  if (end < start) throw new RangeError('end < start');
 
   // Fill 0 bytes; we're done
   if (end === start) return 0;
   if (this.length == 0) return 0;
 
   if (start < 0 || start >= this.length) {
-    throw new Error('start out of bounds');
+    throw new RangeError('start out of bounds');
   }
 
   if (end < 0 || end > this.length) {
-    throw new Error('end out of bounds');
+    throw new RangeError('end out of bounds');
   }
 
   return this.parent.fill(value,
@@ -481,7 +475,7 @@ Buffer.prototype.fill = function fill(value, start, end) {
 
 Buffer.concat = function(list, length) {
   if (!Array.isArray(list)) {
-    throw new Error('Usage: Buffer.concat(list, [length])');
+    throw new TypeError('Usage: Buffer.concat(list, [length])');
   }
 
   if (list.length === 0) {
@@ -513,40 +507,29 @@ Buffer.concat = function(list, length) {
 
 // copy(targetBuffer, targetStart=0, sourceStart=0, sourceEnd=buffer.length)
 Buffer.prototype.copy = function(target, target_start, start, end) {
-  var source = this;
-  start || (start = 0);
-  end || (end = this.length);
-  target_start || (target_start = 0);
-
-  if (end < start) throw new Error('sourceEnd < sourceStart');
+  // set undefined/NaN or out of bounds values equal to their default
+  if (!(target_start >= 0)) target_start = 0;
+  if (!(start >= 0)) start = 0;
+  if (!(end < this.length)) end = this.length;
 
   // Copy 0 bytes; we're done
-  if (end === start) return 0;
-  if (target.length == 0 || source.length == 0) return 0;
+  if (end === start ||
+      target.length === 0 ||
+      this.length === 0 ||
+      start > this.length)
+    return 0;
 
-  if (target_start < 0 || target_start >= target.length) {
-    throw new Error('targetStart out of bounds');
-  }
+  if (end < start)
+    throw new RangeError('sourceEnd < sourceStart');
 
-  if (start < 0 || start >= source.length) {
-    throw new Error('sourceStart out of bounds');
-  }
+  if (target_start >= target.length)
+    throw new RangeError('targetStart out of bounds');
 
-  if (end < 0 || end > source.length) {
-    throw new Error('sourceEnd out of bounds');
-  }
-
-  // Are we oob?
-  if (end > this.length) {
-    end = this.length;
-  }
-
-  if (target.length - target_start < end - start) {
+  if (target.length - target_start < end - start)
     end = target.length - target_start + start;
-  }
 
-  return this.parent.copy(target.parent,
-                          target_start + target.offset,
+  return this.parent.copy(target.parent || target,
+                          target_start + (target.offset || 0),
                           start + this.offset,
                           end + this.offset);
 };
@@ -554,11 +537,10 @@ Buffer.prototype.copy = function(target, target_start, start, end) {
 
 // slice(start, end)
 Buffer.prototype.slice = function(start, end) {
-  if (end === undefined) end = this.length;
-  if (end > this.length) throw new Error('oob');
-  if (start > end) throw new Error('oob');
-  if (start < 0) throw new Error('start out of bounds');
-  return new Buffer(this.parent, end - start, +start + this.offset);
+  var len = this.length;
+  start = clamp(start, len, 0);
+  end = clamp(end, len, len);
+  return new Buffer(this.parent, end - start, start + this.offset);
 };
 
 
@@ -588,35 +570,28 @@ Buffer.prototype.asciiWrite = function(string, offset) {
   return this.write(string, offset, 'ascii');
 };
 
+
+/*
+ * Need to make sure that buffer isn't trying to write out of bounds.
+ * This check is far too slow internally for fast buffers.
+ */
+function checkOffset(offset, ext, length) {
+  if ((offset % 1) !== 0 || offset < 0)
+    throw new RangeError('offset is not uint');
+  if (offset + ext > length)
+    throw new RangeError('Trying to access beyond buffer length');
+}
+
+
 Buffer.prototype.readUInt8 = function(offset, noAssert) {
-  var buffer = this;
-
-  if (!noAssert) {
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
-  return buffer[offset];
+  if (!noAssert)
+    checkOffset(offset, 1, this.length);
+  return this[offset];
 };
 
-function readUInt16(buffer, offset, isBigEndian, noAssert) {
+
+function readUInt16(buffer, offset, isBigEndian) {
   var val = 0;
-
-
-  if (!noAssert) {
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 1 < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
   if (isBigEndian) {
     val = buffer[offset] << 8;
     val |= buffer[offset + 1];
@@ -628,28 +603,24 @@ function readUInt16(buffer, offset, isBigEndian, noAssert) {
   return val;
 }
 
+
 Buffer.prototype.readUInt16LE = function(offset, noAssert) {
+  if (!noAssert)
+    checkOffset(offset, 2, this.length);
   return readUInt16(this, offset, false, noAssert);
 };
 
+
 Buffer.prototype.readUInt16BE = function(offset, noAssert) {
+  if (!noAssert)
+    checkOffset(offset, 2, this.length);
   return readUInt16(this, offset, true, noAssert);
 };
 
+
 function readUInt32(buffer, offset, isBigEndian, noAssert) {
   var val = 0;
 
-  if (!noAssert) {
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 3 < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
   if (isBigEndian) {
     val = buffer[offset + 1] << 16;
     val |= buffer[offset + 2] << 8;
@@ -665,11 +636,17 @@ function readUInt32(buffer, offset, isBigEndian, noAssert) {
   return val;
 }
 
+
 Buffer.prototype.readUInt32LE = function(offset, noAssert) {
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
   return readUInt32(this, offset, false, noAssert);
 };
 
+
 Buffer.prototype.readUInt32BE = function(offset, noAssert) {
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
   return readUInt32(this, offset, true, noAssert);
 };
 
@@ -719,188 +696,107 @@ Buffer.prototype.readUInt32BE = function(offset, noAssert) {
  * (0x007f + 1) * -1
  * (0x0080) * -1
  */
+
 Buffer.prototype.readInt8 = function(offset, noAssert) {
-  var buffer = this;
-  var neg;
-
-  if (!noAssert) {
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
-  neg = buffer[offset] & 0x80;
-  if (!neg) {
-    return (buffer[offset]);
-  }
-
-  return ((0xff - buffer[offset] + 1) * -1);
+  if (!noAssert)
+    checkOffset(offset, 1, this.length);
+  if (!(this[offset] & 0x80))
+    return (this[offset]);
+  return ((0xff - this[offset] + 1) * -1);
 };
 
-function readInt16(buffer, offset, isBigEndian, noAssert) {
-  var neg, val;
 
-  if (!noAssert) {
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
+function readInt16(buffer, offset, isBigEndian) {
+  var val = readUInt16(buffer, offset, isBigEndian);
 
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 1 < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
-  val = readUInt16(buffer, offset, isBigEndian, noAssert);
-  neg = val & 0x8000;
-  if (!neg) {
+  if (!(val & 0x8000))
     return val;
-  }
-
   return (0xffff - val + 1) * -1;
 }
 
+
 Buffer.prototype.readInt16LE = function(offset, noAssert) {
-  return readInt16(this, offset, false, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 2, this.length);
+  return readInt16(this, offset, false);
 };
 
+
 Buffer.prototype.readInt16BE = function(offset, noAssert) {
-  return readInt16(this, offset, true, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 2, this.length);
+  return readInt16(this, offset, true);
 };
 
-function readInt32(buffer, offset, isBigEndian, noAssert) {
-  var neg, val;
 
-  if (!noAssert) {
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
+function readInt32(buffer, offset, isBigEndian) {
+  var val = readUInt32(buffer, offset, isBigEndian);
 
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 3 < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
-  val = readUInt32(buffer, offset, isBigEndian, noAssert);
-  neg = val & 0x80000000;
-  if (!neg) {
+  if (!(val & 0x80000000))
     return (val);
-  }
-
   return (0xffffffff - val + 1) * -1;
 }
 
+
 Buffer.prototype.readInt32LE = function(offset, noAssert) {
-  return readInt32(this, offset, false, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
+  return readInt32(this, offset, false);
 };
 
+
 Buffer.prototype.readInt32BE = function(offset, noAssert) {
-  return readInt32(this, offset, true, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
+  return readInt32(this, offset, true);
 };
 
-function readFloat(buffer, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset + 3 < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
-  return require('buffer_ieee754').readIEEE754(buffer, offset, isBigEndian,
-      23, 4);
-}
-
 Buffer.prototype.readFloatLE = function(offset, noAssert) {
-  return readFloat(this, offset, false, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
+  return this.parent.readFloatLE(this.offset + offset, !!noAssert);
 };
 
+
 Buffer.prototype.readFloatBE = function(offset, noAssert) {
-  return readFloat(this, offset, true, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
+  return this.parent.readFloatBE(this.offset + offset, !!noAssert);
 };
 
-function readDouble(buffer, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset + 7 < buffer.length,
-        'Trying to read beyond buffer length');
-  }
-
-  return require('buffer_ieee754').readIEEE754(buffer, offset, isBigEndian,
-      52, 8);
-}
 
 Buffer.prototype.readDoubleLE = function(offset, noAssert) {
-  return readDouble(this, offset, false, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 8, this.length);
+  return this.parent.readDoubleLE(this.offset + offset, !!noAssert);
 };
 
+
 Buffer.prototype.readDoubleBE = function(offset, noAssert) {
-  return readDouble(this, offset, true, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 8, this.length);
+  return this.parent.readDoubleBE(this.offset + offset, !!noAssert);
 };
 
 
-/*
- * We have to make sure that the value is a valid integer. This means that it is
- * non-negative. It has no fractional component and that it does not exceed the
- * maximum allowed value.
- *
- *      value           The number to check for validity
- *
- *      max             The maximum value
- */
-function verifuint(value, max) {
-  assert.ok(typeof (value) == 'number',
-      'cannot write a non-number as a number');
-
-  assert.ok(value >= 0,
-      'specified a negative value for writing an unsigned value');
-
-  assert.ok(value <= max, 'value is larger than maximum value for type');
-
-  assert.ok(Math.floor(value) === value, 'value has a fractional component');
+function checkInt(buffer, value, offset, ext, max, min) {
+  if ((value % 1) !== 0 || value > max || value < min)
+    throw TypeError('value is out of bounds');
+  if ((offset % 1) !== 0 || offset < 0)
+    throw TypeError('offset is not uint');
+  if (offset + ext > buffer.length || buffer.length + offset < 0)
+    throw RangeError('Trying to write outside buffer length');
 }
 
+
 Buffer.prototype.writeUInt8 = function(value, offset, noAssert) {
-  var buffer = this;
-
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset < buffer.length,
-        'trying to write beyond buffer length');
-
-    verifuint(value, 0xff);
-  }
-
-  buffer[offset] = value;
+  if (!noAssert)
+    checkInt(this, value, offset, 1, 0xff, 0);
+  this[offset] = value;
 };
 
-function writeUInt16(buffer, value, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 1 < buffer.length,
-        'trying to write beyond buffer length');
-
-    verifuint(value, 0xffff);
-  }
 
+function writeUInt16(buffer, value, offset, isBigEndian) {
   if (isBigEndian) {
     buffer[offset] = (value & 0xff00) >>> 8;
     buffer[offset + 1] = value & 0x00ff;
@@ -910,31 +806,22 @@ function writeUInt16(buffer, value, offset, isBigEndian, noAssert) {
   }
 }
 
+
 Buffer.prototype.writeUInt16LE = function(value, offset, noAssert) {
-  writeUInt16(this, value, offset, false, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 2, 0xffff, 0);
+  writeUInt16(this, value, offset, false);
 };
 
+
 Buffer.prototype.writeUInt16BE = function(value, offset, noAssert) {
-  writeUInt16(this, value, offset, true, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 2, 0xffff, 0);
+  writeUInt16(this, value, offset, true);
 };
 
-function writeUInt32(buffer, value, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 3 < buffer.length,
-        'trying to write beyond buffer length');
-
-    verifuint(value, 0xffffffff);
-  }
 
+function writeUInt32(buffer, value, offset, isBigEndian) {
   if (isBigEndian) {
     buffer[offset] = (value >>> 24) & 0xff;
     buffer[offset + 1] = (value >>> 16) & 0xff;
@@ -948,12 +835,18 @@ function writeUInt32(buffer, value, offset, isBigEndian, noAssert) {
   }
 }
 
+
 Buffer.prototype.writeUInt32LE = function(value, offset, noAssert) {
-  writeUInt32(this, value, offset, false, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 4, 0xffffffff, 0);
+  writeUInt32(this, value, offset, false);
 };
 
+
 Buffer.prototype.writeUInt32BE = function(value, offset, noAssert) {
-  writeUInt32(this, value, offset, true, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 4, 0xffffffff, 0);
+  writeUInt32(this, value, offset, true);
 };
 
 
@@ -994,170 +887,69 @@ Buffer.prototype.writeUInt32BE = function(value, offset, noAssert) {
  * hacky, but it should work and get the job done which is our goal here.
  */
 
-/*
- * A series of checks to make sure we actually have a signed 32-bit number
- */
-function verifsint(value, max, min) {
-  assert.ok(typeof (value) == 'number',
-      'cannot write a non-number as a number');
-
-  assert.ok(value <= max, 'value larger than maximum allowed value');
-
-  assert.ok(value >= min, 'value smaller than minimum allowed value');
-
-  assert.ok(Math.floor(value) === value, 'value has a fractional component');
-}
-
-function verifIEEE754(value, max, min) {
-  assert.ok(typeof (value) == 'number',
-      'cannot write a non-number as a number');
-
-  assert.ok(value <= max, 'value larger than maximum allowed value');
-
-  assert.ok(value >= min, 'value smaller than minimum allowed value');
-}
-
 Buffer.prototype.writeInt8 = function(value, offset, noAssert) {
-  var buffer = this;
-
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset < buffer.length,
-        'Trying to write beyond buffer length');
-
-    verifsint(value, 0x7f, -0x80);
-  }
-
-  if (value >= 0) {
-    buffer.writeUInt8(value, offset, noAssert);
-  } else {
-    buffer.writeUInt8(0xff + value + 1, offset, noAssert);
-  }
+  if (!noAssert)
+    checkInt(this, value, offset, 1, 0x7f, -0x80);
+  if (value < 0) value = 0xff + value + 1;
+  this[offset] = value;
 };
 
-function writeInt16(buffer, value, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 1 < buffer.length,
-        'Trying to write beyond buffer length');
-
-    verifsint(value, 0x7fff, -0x8000);
-  }
-
-  if (value >= 0) {
-    writeUInt16(buffer, value, offset, isBigEndian, noAssert);
-  } else {
-    writeUInt16(buffer, 0xffff + value + 1, offset, isBigEndian, noAssert);
-  }
-}
 
 Buffer.prototype.writeInt16LE = function(value, offset, noAssert) {
-  writeInt16(this, value, offset, false, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 2, 0x7fff, -0x8000);
+  if (value < 0) value = 0xffff + value + 1;
+  writeUInt16(this, value, offset, false);
 };
 
+
 Buffer.prototype.writeInt16BE = function(value, offset, noAssert) {
-  writeInt16(this, value, offset, true, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 2, 0x7fff, -0x8000);
+  if (value < 0) value = 0xffff + value + 1;
+  writeUInt16(this, value, offset, true);
 };
 
-function writeInt32(buffer, value, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 3 < buffer.length,
-        'Trying to write beyond buffer length');
-
-    verifsint(value, 0x7fffffff, -0x80000000);
-  }
-
-  if (value >= 0) {
-    writeUInt32(buffer, value, offset, isBigEndian, noAssert);
-  } else {
-    writeUInt32(buffer, 0xffffffff + value + 1, offset, isBigEndian, noAssert);
-  }
-}
 
 Buffer.prototype.writeInt32LE = function(value, offset, noAssert) {
-  writeInt32(this, value, offset, false, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 4, 0x7fffffff, -0x80000000);
+  if (value < 0) value = 0xffffffff + value + 1;
+  writeUInt32(this, value, offset, false);
 };
 
+
 Buffer.prototype.writeInt32BE = function(value, offset, noAssert) {
-  writeInt32(this, value, offset, true, noAssert);
+  if (!noAssert)
+    checkInt(this, value, offset, 4, 0x7fffffff, -0x80000000);
+  if (value < 0) value = 0xffffffff + value + 1;
+  writeUInt32(this, value, offset, true);
 };
 
-function writeFloat(buffer, value, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 3 < buffer.length,
-        'Trying to write beyond buffer length');
-
-    verifIEEE754(value, 3.4028234663852886e+38, -3.4028234663852886e+38);
-  }
-
-  require('buffer_ieee754').writeIEEE754(buffer, value, offset, isBigEndian,
-      23, 4);
-}
 
 Buffer.prototype.writeFloatLE = function(value, offset, noAssert) {
-  writeFloat(this, value, offset, false, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
+  this.parent.writeFloatLE(value, this.offset + offset, !!noAssert);
 };
 
+
 Buffer.prototype.writeFloatBE = function(value, offset, noAssert) {
-  writeFloat(this, value, offset, true, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 4, this.length);
+  this.parent.writeFloatBE(value, this.offset + offset, !!noAssert);
 };
 
-function writeDouble(buffer, value, offset, isBigEndian, noAssert) {
-  if (!noAssert) {
-    assert.ok(value !== undefined && value !== null,
-        'missing value');
-
-    assert.ok(typeof (isBigEndian) === 'boolean',
-        'missing or invalid endian');
-
-    assert.ok(offset !== undefined && offset !== null,
-        'missing offset');
-
-    assert.ok(offset + 7 < buffer.length,
-        'Trying to write beyond buffer length');
-
-    verifIEEE754(value, 1.7976931348623157E+308, -1.7976931348623157E+308);
-  }
-
-  require('buffer_ieee754').writeIEEE754(buffer, value, offset, isBigEndian,
-      52, 8);
-}
 
 Buffer.prototype.writeDoubleLE = function(value, offset, noAssert) {
-  writeDouble(this, value, offset, false, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 8, this.length);
+  this.parent.writeDoubleLE(value, this.offset + offset, !!noAssert);
 };
 
+
 Buffer.prototype.writeDoubleBE = function(value, offset, noAssert) {
-  writeDouble(this, value, offset, true, noAssert);
+  if (!noAssert)
+    checkOffset(offset, 8, this.length);
+  this.parent.writeDoubleBE(value, this.offset + offset, !!noAssert);
 };
