@@ -27,8 +27,6 @@ var EventEmitter = require('events').EventEmitter;
 var FreeList = require('freelist').FreeList;
 var HTTPParser = process.binding('http_parser').HTTPParser;
 var assert = require('assert').ok;
-var END_OF_FILE = {};
-
 
 var debug;
 if (process.env.NODE_DEBUG && /http/.test(process.env.NODE_DEBUG)) {
@@ -37,6 +35,16 @@ if (process.env.NODE_DEBUG && /http/.test(process.env.NODE_DEBUG)) {
   debug = function() { };
 }
 
+function readStart(socket) {
+  if (!socket || !socket._handle || !socket._handle.readStart) return;
+  socket._handle.readStart();
+}
+
+function readStop(socket) {
+  if (!socket || !socket._handle || !socket._handle.readStop) return;
+  socket._handle.readStop();
+}
+
 // Only called in the slow case where slow means
 // that the request headers were either fragmented
 // across multiple TCP packets or too large to be
@@ -114,45 +122,58 @@ function parserOnHeadersComplete(info) {
   return skipBody;
 }
 
+// XXX This is a mess.
+// TODO: http.Parser should be a Writable emits request/response events.
 function parserOnBody(b, start, len) {
   var parser = this;
-  var slice = b.slice(start, start + len);
-  if (parser.incoming._paused || parser.incoming._pendings.length) {
-    parser.incoming._pendings.push(slice);
-  } else {
-    parser.incoming._emitData(slice);
+  var stream = parser.incoming;
+
+  // if the stream has already been removed, then drop it.
+  if (!stream)
+    return;
+
+  var socket = stream.socket;
+
+  // pretend this was the result of a stream._read call.
+  if (len > 0 && !stream._dumped) {
+    var slice = b.slice(start, start + len);
+    var ret = stream.push(slice);
+    if (!ret)
+      readStop(socket);
   }
 }
 
 function parserOnMessageComplete() {
   var parser = this;
-  parser.incoming.complete = true;
+  var stream = parser.incoming;
 
-  // Emit any trailing headers.
-  var headers = parser._headers;
-  if (headers) {
-    for (var i = 0, n = headers.length; i < n; i += 2) {
-      var k = headers[i];
-      var v = headers[i + 1];
-      parser.incoming._addHeaderLine(k, v);
+  if (stream) {
+    stream.complete = true;
+    // Emit any trailing headers.
+    var headers = parser._headers;
+    if (headers) {
+      for (var i = 0, n = headers.length; i < n; i += 2) {
+        var k = headers[i];
+        var v = headers[i + 1];
+        parser.incoming._addHeaderLine(k, v);
+      }
+      parser._headers = [];
+      parser._url = '';
     }
-    parser._headers = [];
-    parser._url = '';
+
+    if (!stream.upgrade)
+      // For upgraded connections, also emit this after parser.execute
+      stream.push(null);
   }
 
-  if (!parser.incoming.upgrade) {
-    // For upgraded connections, also emit this after parser.execute
-    if (parser.incoming._paused || parser.incoming._pendings.length) {
-      parser.incoming._pendings.push(END_OF_FILE);
-    } else {
-      parser.incoming.readable = false;
-      parser.incoming._emitEnd();
-    }
+  if (stream && !parser.incoming._pendings.length) {
+    // For emit end event
+    stream.push(null);
   }
 
   if (parser.socket.readable) {
     // force to read the next incoming message
-    parser.socket.resume();
+    readStart(parser.socket);
   }
 }
 
@@ -230,7 +251,7 @@ var STATUS_CODES = exports.STATUS_CODES = {
   502 : 'Bad Gateway',
   503 : 'Service Unavailable',
   504 : 'Gateway Time-out',
-  505 : 'HTTP Version not supported',
+  505 : 'HTTP Version Not Supported',
   506 : 'Variant Also Negotiates',    // RFC 2295
   507 : 'Insufficient Storage',       // RFC 4918
   509 : 'Bandwidth Limit Exceeded',
@@ -263,9 +284,12 @@ function utcDate() {
 
 /* Abstract base class for ServerRequest and ClientResponse. */
 function IncomingMessage(socket) {
-  Stream.call(this);
+  Stream.Readable.call(this);
+
+  // XXX This implementation is kind of all over the place
+  // When the parser emits body chunks, they go in this list.
+  // _read() pulls them out, and when it finds EOF, it ends.
 
-  // TODO Remove one of these eventually.
   this.socket = socket;
   this.connection = socket;
 
@@ -276,97 +300,61 @@ function IncomingMessage(socket) {
 
   this.readable = true;
 
-  this._paused = false;
   this._pendings = [];
-
-  this._endEmitted = false;
+  this._pendingIndex = 0;
 
   // request (server) only
   this.url = '';
-
   this.method = null;
 
   // response (client) only
   this.statusCode = null;
   this.client = this.socket;
+
+  // flag for backwards compatibility grossness.
+  this._consuming = false;
+
+  // flag for when we decide that this message cannot possibly be
+  // read by the user, so there's no point continuing to handle it.
+  this._dumped = false;
 }
-util.inherits(IncomingMessage, Stream);
+util.inherits(IncomingMessage, Stream.Readable);
 
 
 exports.IncomingMessage = IncomingMessage;
 
 
+IncomingMessage.prototype.setTimeout = function(msecs, callback) {
+  if (callback)
+    this.on('timeout', callback);
+  this.socket.setTimeout(msecs);
+};
+
+
+IncomingMessage.prototype.read = function(n) {
+  this._consuming = true;
+  this.read = Stream.Readable.prototype.read;
+  return this.read(n);
+};
+
+
+IncomingMessage.prototype._read = function(n) {
+  // We actually do almost nothing here, because the parserOnBody
+  // function fills up our internal buffer directly.  However, we
+  // do need to unpause the underlying socket so that it flows.
+  if (!this.socket.readable)
+    this.push(null);
+  else
+    readStart(this.socket);
+};
+
+
+// It's possible that the socket will be destroyed, and removed from
+// any messages, before ever calling this.  In that case, just skip
+// it, since something else is destroying this connection anyway.
 IncomingMessage.prototype.destroy = function(error) {
-  this.socket.destroy(error);
-};
-
-
-IncomingMessage.prototype.setEncoding = function(encoding) {
-  var StringDecoder = require('string_decoder').StringDecoder; // lazy load
-  this._decoder = new StringDecoder(encoding);
-};
-
-
-IncomingMessage.prototype.pause = function() {
-  this._paused = true;
-  this.socket.pause();
-};
-
-
-IncomingMessage.prototype.resume = function() {
-  this._paused = false;
-  if (this.socket) {
-    this.socket.resume();
-  }
-
-  this._emitPending();
-};
-
-
-IncomingMessage.prototype._emitPending = function(callback) {
-  if (this._pendings.length) {
-    var self = this;
-    process.nextTick(function() {
-      while (!self._paused && self._pendings.length) {
-        var chunk = self._pendings.shift();
-        if (chunk !== END_OF_FILE) {
-          assert(Buffer.isBuffer(chunk));
-          self._emitData(chunk);
-        } else {
-          assert(self._pendings.length === 0);
-          self.readable = false;
-          self._emitEnd();
-        }
-      }
-
-      if (callback) {
-        callback();
-      }
-    });
-  } else if (callback) {
-    callback();
-  }
-};
-
-
-IncomingMessage.prototype._emitData = function(d) {
-  if (this._decoder) {
-    var string = this._decoder.write(d);
-    if (string.length) {
-      this.emit('data', string);
-    }
-  } else {
-    this.emit('data', d);
-  }
-};
-
-
-IncomingMessage.prototype._emitEnd = function() {
-  if (!this._endEmitted) {
-    this.emit('end');
-  }
-
-  this._endEmitted = true;
+  if (this.socket)
+    this.socket.destroy(error);
 };
 
 
@@ -384,7 +372,7 @@ IncomingMessage.prototype._addHeaderLine = function(field, value) {
   switch (field) {
     // Array headers:
     case 'set-cookie':
-      if (field in dest) {
+      if (dest[field] !== undefined) {
         dest[field].push(value);
       } else {
         dest[field] = [value];
@@ -404,7 +392,7 @@ IncomingMessage.prototype._addHeaderLine = function(field, value) {
     case 'proxy-authenticate':
     case 'sec-websocket-extensions':
     case 'sec-websocket-protocol':
-      if (field in dest) {
+      if (dest[field] !== undefined) {
         dest[field] += ', ' + value;
       } else {
         dest[field] = value;
@@ -415,20 +403,33 @@ IncomingMessage.prototype._addHeaderLine = function(field, value) {
     default:
       if (field.slice(0, 2) == 'x-') {
         // except for x-
-        if (field in dest) {
+        if (dest[field] !== undefined) {
           dest[field] += ', ' + value;
         } else {
           dest[field] = value;
         }
       } else {
         // drop duplicates
-        if (!(field in dest)) dest[field] = value;
+        if (dest[field] === undefined) dest[field] = value;
       }
       break;
   }
 };
 
 
+// Call this instead of resume() if we want to just
+// dump all the data to /dev/null
+IncomingMessage.prototype._dump = function() {
+  if (!this._dumped) {
+    this._dumped = true;
+    if (this.socket.parser) this.socket.parser.incoming = null;
+    this.push(null);
+    readStart(this.socket);
+    this.read();
+  }
+};
+
+
 function OutgoingMessage() {
   Stream.call(this);
 
@@ -447,6 +448,10 @@ function OutgoingMessage() {
   this._trailer = '';
 
   this.finished = false;
+  this._hangupClose = false;
+
+  this.socket = null;
+  this.connection = null;
 }
 util.inherits(OutgoingMessage, Stream);
 
@@ -454,8 +459,28 @@ util.inherits(OutgoingMessage, Stream);
 exports.OutgoingMessage = OutgoingMessage;
 
 
+OutgoingMessage.prototype.setTimeout = function(msecs, callback) {
+  if (callback)
+    this.on('timeout', callback);
+  if (!this.socket) {
+    this.once('socket', function(socket) {
+      socket.setTimeout(msecs);
+    });
+  } else
+    this.socket.setTimeout(msecs);
+};
+
+
+// It's possible that the socket will be destroyed, and removed from
+// any messages, before ever calling this.  In that case, just skip
+// it, since something else is destroying this connection anyway.
 OutgoingMessage.prototype.destroy = function(error) {
-  this.socket.destroy(error);
+  if (this.socket)
+    this.socket.destroy(error);
+  else
+    this.once('socket', function(socket) {
+      socket.destroy(error);
+    });
 };
 
 
@@ -484,7 +509,8 @@ OutgoingMessage.prototype._writeRaw = function(data, encoding) {
 
   if (this.connection &&
       this.connection._httpMessage === this &&
-      this.connection.writable) {
+      this.connection.writable &&
+      !this.connection.destroyed) {
     // There might be pending data in the this.output buffer.
     while (this.output.length) {
       if (!this.connection.writable) {
@@ -498,7 +524,12 @@ OutgoingMessage.prototype._writeRaw = function(data, encoding) {
 
     // Directly write to socket.
     return this.connection.write(data, encoding);
+  } else if (this.connection && this.connection.destroyed) {
+    // The socket was destroyed.  If we're still trying to write to it,
+    // then we haven't gotten the 'close' event yet.
+    return false;
   } else {
+    // buffer, as long as we're not destroyed.
     this._buffer(data, encoding);
     return false;
   }
@@ -533,42 +564,20 @@ OutgoingMessage.prototype._buffer = function(data, encoding) {
 
 
 OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
-  var sentConnectionHeader = false;
-  var sentContentLengthHeader = false;
-  var sentTransferEncodingHeader = false;
-  var sentDateHeader = false;
-  var sentExpect = false;
-
   // firstLine in the case of request is: 'GET /index.html HTTP/1.1\r\n'
   // in the case of response it is: 'HTTP/1.1 200 OK\r\n'
-  var messageHeader = firstLine;
+  var state = {
+    sentConnectionHeader: false,
+    sentContentLengthHeader: false,
+    sentTransferEncodingHeader: false,
+    sentDateHeader: false,
+    sentExpect: false,
+    messageHeader: firstLine
+  };
+
   var field, value;
   var self = this;
 
-  function store(field, value) {
-    messageHeader += field + ': ' + value + CRLF;
-
-    if (connectionExpression.test(field)) {
-      sentConnectionHeader = true;
-      if (closeExpression.test(value)) {
-        self._last = true;
-      } else {
-        self.shouldKeepAlive = true;
-      }
-
-    } else if (transferEncodingExpression.test(field)) {
-      sentTransferEncodingHeader = true;
-      if (chunkExpression.test(value)) self.chunkedEncoding = true;
-
-    } else if (contentLengthExpression.test(field)) {
-      sentContentLengthHeader = true;
-    } else if (dateExpression.test(field)) {
-      sentDateHeader = true;
-    } else if (expectExpression.test(field)) {
-      sentExpect = true;
-    }
-  }
-
   if (headers) {
     var keys = Object.keys(headers);
     var isArray = (Array.isArray(headers));
@@ -586,37 +595,58 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
 
       if (Array.isArray(value)) {
         for (var j = 0; j < value.length; j++) {
-          store(field, value[j]);
+          storeHeader(this, state, field, value[j]);
         }
       } else {
-        store(field, value);
+        storeHeader(this, state, field, value);
       }
     }
   }
 
   // Date header
-  if (this.sendDate == true && sentDateHeader == false) {
-    messageHeader += 'Date: ' + utcDate() + CRLF;
+  if (this.sendDate == true && state.sentDateHeader == false) {
+    state.messageHeader += 'Date: ' + utcDate() + CRLF;
+  }
+
+  // Force the connection to close when the response is a 204 No Content or
+  // a 304 Not Modified and the user has set a "Transfer-Encoding: chunked"
+  // header.
+  //
+  // RFC 2616 mandates that 204 and 304 responses MUST NOT have a body but
+  // node.js used to send out a zero chunk anyway to accommodate clients
+  // that don't have special handling for those responses.
+  //
+  // It was pointed out that this might confuse reverse proxies to the point
+  // of creating security liabilities, so suppress the zero chunk and force
+  // the connection to close.
+  var statusCode = this.statusCode;
+  if ((statusCode == 204 || statusCode === 304) &&
+      this.chunkedEncoding === true) {
+    debug(statusCode + ' response should not use chunked encoding,' +
+          ' closing connection.');
+    this.chunkedEncoding = false;
+    this.shouldKeepAlive = false;
   }
 
   // keep-alive logic
-  if (sentConnectionHeader === false) {
+  if (state.sentConnectionHeader === false) {
     var shouldSendKeepAlive = this.shouldKeepAlive &&
-        (sentContentLengthHeader ||
+        (state.sentContentLengthHeader ||
          this.useChunkedEncodingByDefault ||
          this.agent);
     if (shouldSendKeepAlive) {
-      messageHeader += 'Connection: keep-alive\r\n';
+      state.messageHeader += 'Connection: keep-alive\r\n';
     } else {
       this._last = true;
-      messageHeader += 'Connection: close\r\n';
+      state.messageHeader += 'Connection: close\r\n';
     }
   }
 
-  if (sentContentLengthHeader == false && sentTransferEncodingHeader == false) {
+  if (state.sentContentLengthHeader == false &&
+      state.sentTransferEncodingHeader == false) {
     if (this._hasBody) {
       if (this.useChunkedEncodingByDefault) {
-        messageHeader += 'Transfer-Encoding: chunked\r\n';
+        state.messageHeader += 'Transfer-Encoding: chunked\r\n';
         this.chunkedEncoding = true;
       } else {
         this._last = true;
@@ -627,14 +657,43 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
     }
   }
 
-  this._header = messageHeader + CRLF;
+  this._header = state.messageHeader + CRLF;
   this._headerSent = false;
 
   // wait until the first body chunk, or close(), is sent to flush,
   // UNLESS we're sending Expect: 100-continue.
-  if (sentExpect) this._send('');
+  if (state.sentExpect) this._send('');
 };
 
+function storeHeader(self, state, field, value) {
+  // Protect against response splitting. The if statement is there to
+  // minimize the performance impact in the common case.
+  if (/[\r\n]/.test(value))
+    value = value.replace(/[\r\n]+[ \t]*/g, '');
+
+  state.messageHeader += field + ': ' + value + CRLF;
+
+  if (connectionExpression.test(field)) {
+    state.sentConnectionHeader = true;
+    if (closeExpression.test(value)) {
+      self._last = true;
+    } else {
+      self.shouldKeepAlive = true;
+    }
+
+  } else if (transferEncodingExpression.test(field)) {
+    state.sentTransferEncodingHeader = true;
+    if (chunkExpression.test(value)) self.chunkedEncoding = true;
+
+  } else if (contentLengthExpression.test(field)) {
+    state.sentContentLengthHeader = true;
+  } else if (dateExpression.test(field)) {
+    state.sentDateHeader = true;
+  } else if (expectExpression.test(field)) {
+    state.sentExpect = true;
+  }
+}
+
 
 OutgoingMessage.prototype.setHeader = function(name, value) {
   if (arguments.length < 2) {
@@ -699,6 +758,12 @@ OutgoingMessage.prototype._renderHeaders = function() {
 };
 
 
+Object.defineProperty(OutgoingMessage.prototype, 'headersSent', {
+  configurable: true,
+  enumerable: true,
+  get: function() { return !!this._header; }
+});
+
 
 OutgoingMessage.prototype.write = function(chunk, encoding) {
   if (!this._header) {
@@ -715,19 +780,40 @@ OutgoingMessage.prototype.write = function(chunk, encoding) {
     throw new TypeError('first argument must be a string or Buffer');
   }
 
-  if (chunk.length === 0) return false;
+  // If we get an empty string or buffer, then just do nothing, and
+  // signal the user to keep writing.
+  if (chunk.length === 0) return true;
+
+  // TODO(bnoordhuis) Temporary optimization hack, remove in v0.11. We only
+  // want to convert the buffer when we're sending:
+  //
+  //   a) Transfer-Encoding chunks, because it lets us pack the chunk header
+  //      and the chunk into a single write(), or
+  //
+  //   b) the first chunk of a fixed-length request, because it lets us pack
+  //      the request headers and the chunk into a single write().
+  //
+  // Converting to strings is expensive, CPU-wise, but reducing the number
+  // of write() calls more than makes up for that because we're dramatically
+  // reducing the number of TCP roundtrips.
+  if (chunk instanceof Buffer && (this.chunkedEncoding || !this._headerSent)) {
+    chunk = chunk.toString('binary');
+    encoding = 'binary';
+  }
 
   var len, ret;
   if (this.chunkedEncoding) {
-    if (typeof(chunk) === 'string') {
+    if (typeof(chunk) === 'string' &&
+        encoding !== 'hex' &&
+        encoding !== 'base64') {
       len = Buffer.byteLength(chunk, encoding);
       chunk = len.toString(16) + CRLF + chunk + CRLF;
       ret = this._send(chunk, encoding);
     } else {
-      // buffer
+      // buffer, or a non-toString-friendly encoding
       len = chunk.length;
       this._send(len.toString(16) + CRLF);
-      this._send(chunk);
+      this._send(chunk, encoding);
       ret = this._send(CRLF);
     }
   } else {
@@ -786,6 +872,14 @@ OutgoingMessage.prototype.end = function(data, encoding) {
             this.connection.writable &&
             this.connection._httpMessage === this;
 
+  // The benefits of the hot-path optimization below start to fall
+  // off when the buffer size gets up near 128KB, because the cost
+  // of the copy is more than the cost of the extra write() call.
+  // Switch to the write/end method at that point.  Heuristics and
+  // magic numbers are awful, but slow http responses are worse.
+  if (hot && Buffer.isBuffer(data) && data.length > 120 * 1024)
+    hot = false;
+
   if (hot) {
     // Hot path. They're doing
     //   res.writeHead();
@@ -893,9 +987,11 @@ OutgoingMessage.prototype._finish = function() {
   assert(this.connection);
   if (this instanceof ServerResponse) {
     DTRACE_HTTP_SERVER_RESPONSE(this.connection);
+    COUNTER_HTTP_SERVER_RESPONSE();
   } else {
     assert(this instanceof ClientRequest);
     DTRACE_HTTP_CLIENT_REQUEST(this, this.connection);
+    COUNTER_HTTP_CLIENT_REQUEST();
   }
   this.emit('finish');
 };
@@ -993,6 +1089,7 @@ ServerResponse.prototype.assignSocket = function(socket) {
   socket.on('close', onServerResponseClose);
   this.socket = socket;
   this.connection = socket;
+  this.emit('socket', socket);
   this._flush();
 };
 
@@ -1036,7 +1133,7 @@ ServerResponse.prototype.writeHead = function(statusCode) {
       var field;
       for (var i = 0, len = obj.length; i < len; ++i) {
         field = obj[i][0];
-        if (field in headers) {
+        if (headers[field] !== undefined) {
           obj.push([field, headers[field]]);
         }
       }
@@ -1078,7 +1175,7 @@ ServerResponse.prototype.writeHead = function(statusCode) {
 
   // don't keep alive connections where the client expects 100 Continue
   // but we sent a final status; they may put extra bytes on the wire.
-  if (this._expect_continue && ! this._sent100) {
+  if (this._expect_continue && !this._sent100) {
     this.shouldKeepAlive = false;
   }
 
@@ -1116,7 +1213,8 @@ function Agent(options) {
       name += ':' + localAddress;
     }
 
-    if (self.requests[name] && self.requests[name].length) {
+    if (!socket.destroyed &&
+        self.requests[name] && self.requests[name].length) {
       self.requests[name].shift().onSocket(socket);
       if (self.requests[name].length === 0) {
         // don't leak
@@ -1383,18 +1481,26 @@ function socketCloseListener() {
     // Socket closed before we emitted 'end' below.
     req.res.emit('aborted');
     var res = req.res;
-    req.res._emitPending(function() {
-      res._emitEnd();
+    res.on('end', function() {
       res.emit('close');
-      res = null;
     });
+    res.push(null);
   } else if (!req.res && !req._hadError) {
     // This socket error fired before we started to
     // receive a response. The error needs to
     // fire on the request.
     req.emit('error', createHangUpError());
+    req._hadError = true;
   }
 
+  // Too bad.  That output wasn't getting written.
+  // This is pretty terrible that it doesn't raise an error.
+  // Fixed better in v0.10
+  if (req.output)
+    req.output.length = 0;
+  if (req.outputEncodings)
+    req.outputEncodings.length = 0;
+
   if (parser) {
     parser.finish();
     freeParser(parser, req);
@@ -1465,7 +1571,7 @@ function socketOnData(d, start, end) {
     var bodyHead = d.slice(start + bytesParsed, end);
 
     var eventName = req.method === 'CONNECT' ? 'connect' : 'upgrade';
-    if (req.listeners(eventName).length) {
+    if (EventEmitter.listenerCount(req, eventName) > 0) {
       req.upgradeOrConnect = true;
 
       // detach the socket
@@ -1490,11 +1596,13 @@ function socketOnData(d, start, end) {
 }
 
 
+// client
 function parserOnIncomingClient(res, shouldKeepAlive) {
   var parser = this;
   var socket = this.socket;
   var req = socket._httpMessage;
 
+
   // propogate "domain" setting...
   if (req.domain && !res.domain) {
     debug('setting "res.domain"');
@@ -1541,15 +1649,24 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
 
 
   DTRACE_HTTP_CLIENT_RESPONSE(socket, req);
-  req.emit('response', res);
+  COUNTER_HTTP_CLIENT_RESPONSE();
   req.res = res;
   res.req = req;
 
+  // add our listener first, so that we guarantee socket cleanup
   res.on('end', responseOnEnd);
+  var handled = req.emit('response', res);
+
+  // If the user did not listen for the 'response' event, then they
+  // can't possibly read the data, so we ._dump() it into the void
+  // so that the socket doesn't hang there in a paused state.
+  if (!handled)
+    res._dump();
 
   return isHeadResponse;
 }
 
+// client
 function responseOnEnd() {
   var res = this;
   var req = res.req;
@@ -1569,7 +1686,11 @@ function responseOnEnd() {
     }
     socket.removeListener('close', socketCloseListener);
     socket.removeListener('error', socketErrorListener);
-    socket.emit('free');
+    // Mark this socket as available, AFTER user-added end
+    // handlers have a chance to run.
+    process.nextTick(function() {
+      socket.emit('free');
+    });
   }
 }
 
@@ -1654,15 +1775,18 @@ ClientRequest.prototype.setTimeout = function(msecs, callback) {
     return;
   }
 
+  // Set timeoutCb so that it'll get cleaned up on request end
+  this.timeoutCb = emitTimeout;
   if (this.socket) {
+    var sock = this.socket;
     this.socket.once('connect', function() {
-      this.setTimeout(msecs, emitTimeout);
+      sock.setTimeout(msecs, emitTimeout);
     });
     return;
   }
 
   this.once('socket', function(sock) {
-    this.setTimeout(msecs, emitTimeout);
+    sock.setTimeout(msecs, emitTimeout);
   });
 };
 
@@ -1721,10 +1845,23 @@ function Server(requestListener) {
   this.httpAllowHalfOpen = false;
 
   this.addListener('connection', connectionListener);
+
+  this.addListener('clientError', function(err, conn) {
+    conn.destroy(err);
+  });
+
+  this.timeout = 2 * 60 * 1000;
 }
 util.inherits(Server, net.Server);
 
 
+Server.prototype.setTimeout = function(msecs, callback) {
+  this.timeout = msecs;
+  if (callback)
+    this.on('timeout', callback);
+};
+
+
 exports.Server = Server;
 
 
@@ -1750,7 +1887,8 @@ function connectionListener(socket) {
   function serverSocketCloseListener() {
     debug('server socket close');
     // mark this parser as reusable
-    freeParser(parser);
+    if (this.parser)
+      freeParser(this.parser);
 
     abortIncoming();
   }
@@ -1759,9 +1897,20 @@ function connectionListener(socket) {
 
   httpSocketSetup(socket);
 
-  socket.setTimeout(2 * 60 * 1000); // 2 minute timeout
-  socket.once('timeout', function() {
-    socket.destroy();
+  // If the user has added a listener to the server,
+  // request, or response, then it's their responsibility.
+  // otherwise, destroy on timeout by default
+  if (self.timeout)
+    socket.setTimeout(self.timeout);
+  socket.on('timeout', function() {
+    var req = socket.parser && socket.parser.incoming;
+    var reqTimeout = req && !req.complete && req.emit('timeout', socket);
+    var res = socket._httpMessage;
+    var resTimeout = res && res.emit('timeout', socket);
+    var serverTimeout = self.emit('timeout', socket);
+
+    if (!reqTimeout && !resTimeout && !serverTimeout)
+      socket.destroy();
   });
 
   var parser = parsers.alloc();
@@ -1779,7 +1928,7 @@ function connectionListener(socket) {
   }
 
   socket.addListener('error', function(e) {
-    self.emit('clientError', e);
+    self.emit('clientError', e, this);
   });
 
   socket.ondata = function(d, start, end) {
@@ -1802,7 +1951,7 @@ function connectionListener(socket) {
       var bodyHead = d.slice(start + bytesParsed, end);
 
       var eventName = req.method === 'CONNECT' ? 'connect' : 'upgrade';
-      if (self.listeners(eventName).length) {
+      if (EventEmitter.listenerCount(self, eventName) > 0) {
         self.emit(eventName, req, req.socket, bodyHead);
       } else {
         // Got upgrade header or CONNECT method, but have no handler.
@@ -1841,9 +1990,10 @@ function connectionListener(socket) {
     incoming.push(req);
 
     var res = new ServerResponse(req);
-    debug('server response shouldKeepAlive: ' + shouldKeepAlive);
+
     res.shouldKeepAlive = shouldKeepAlive;
     DTRACE_HTTP_SERVER_REQUEST(req, socket);
+    COUNTER_HTTP_SERVER_REQUEST();
 
     if (socket._httpMessage) {
       // There are already pending outgoing res, append.
@@ -1862,6 +2012,12 @@ function connectionListener(socket) {
 
       incoming.shift();
 
+      // if the user never called req.read(), and didn't pipe() or
+      // .resume() or .on('data'), then we call req._dump() so that the
+      // bytes will be pulled off the wire.
+      if (!req._consuming)
+        req._dump();
+
       res.detachSocket(socket);
 
       if (res._last) {
@@ -1875,11 +2031,11 @@ function connectionListener(socket) {
       }
     });
 
-    if ('expect' in req.headers &&
+    if (req.headers.expect !== undefined &&
         (req.httpVersionMajor == 1 && req.httpVersionMinor == 1) &&
         continueExpression.test(req.headers['expect'])) {
       res._expect_continue = true;
-      if (self.listeners('checkContinue').length) {
+      if (EventEmitter.listenerCount(self, 'checkContinue') > 0) {
         self.emit('checkContinue', req, res);
       } else {
         res.writeContinue();
@@ -1929,6 +2085,11 @@ Client.prototype.request = function(method, path, headers) {
   // but it will get removed when we remove this legacy interface.
   c.on('socket', function(s) {
     s.on('end', function() {
+      if (self._decoder) {
+        var ret = self._decoder.end();
+        if (ret)
+          self.emit('data', ret);
+      }
       self.emit('end');
     });
   });
