@@ -49,6 +49,8 @@ function Interface(input, output, completer, terminal) {
     return new Interface(input, output, completer, terminal);
   }
 
+  this._sawReturn = false;
+
   EventEmitter.call(this);
 
   if (arguments.length === 1) {
@@ -75,7 +77,6 @@ function Interface(input, output, completer, terminal) {
 
   this.output = output;
   this.input = input;
-  input.resume();
 
   // Check arity, 2 - for async, 1 for sync
   this.completer = completer.length === 2 ? completer : function(v, callback) {
@@ -86,12 +87,28 @@ function Interface(input, output, completer, terminal) {
 
   this.terminal = !!terminal;
 
+  function ondata(data) {
+    self._normalWrite(data);
+  }
+
+  function onend() {
+    self.close();
+  }
+
+  function onkeypress(s, key) {
+    self._ttyWrite(s, key);
+  }
+
+  function onresize() {
+    self._refreshLine();
+  }
+
   if (!this.terminal) {
-    input.on('data', function(data) {
-      self._normalWrite(data);
-    });
-    input.on('end', function() {
-      self.close();
+    input.on('data', ondata);
+    input.on('end', onend);
+    self.once('close', function() {
+      input.removeListener('data', ondata);
+      input.removeListener('end', onend);
     });
     var StringDecoder = require('string_decoder').StringDecoder; // lazy load
     this._decoder = new StringDecoder('utf8');
@@ -101,9 +118,7 @@ function Interface(input, output, completer, terminal) {
     exports.emitKeypressEvents(input);
 
     // input usually refers to stdin
-    input.on('keypress', function(s, key) {
-      self._ttyWrite(s, key);
-    });
+    input.on('keypress', onkeypress);
 
     // Current line
     this.line = '';
@@ -117,10 +132,14 @@ function Interface(input, output, completer, terminal) {
     this.history = [];
     this.historyIndex = -1;
 
-    output.on('resize', function() {
-      self._refreshLine();
+    output.on('resize', onresize);
+    self.once('close', function() {
+      input.removeListener('keypress', onkeypress);
+      output.removeListener('resize', onresize);
     });
   }
+
+  input.resume();
 }
 
 inherits(Interface, EventEmitter);
@@ -188,12 +207,14 @@ Interface.prototype._onLine = function(line) {
 Interface.prototype._addHistory = function() {
   if (this.line.length === 0) return '';
 
-  this.history.unshift(this.line);
+  if (this.history.length === 0 || this.history[0] !== this.line) {
+    this.history.unshift(this.line);
+
+    // Only store so many
+    if (this.history.length > kHistorySize) this.history.pop();
+  }
+
   this.historyIndex = -1;
-
-  // Only store so many
-  if (this.history.length > kHistorySize) this.history.pop();
-
   return this.history[0];
 };
 
@@ -273,23 +294,32 @@ Interface.prototype.write = function(d, key) {
   this.terminal ? this._ttyWrite(d, key) : this._normalWrite(d);
 };
 
+// \r\n, \n, or \r followed by something other than \n
+var lineEnding = /\r?\n|\r(?!\n)/;
 Interface.prototype._normalWrite = function(b) {
   if (b === undefined) {
     return;
   }
   var string = this._decoder.write(b);
+  if (this._sawReturn) {
+    string = string.replace(/^\n/, '');
+    this._sawReturn = false;
+  }
+
   if (this._line_buffer) {
     string = this._line_buffer + string;
     this._line_buffer = null;
   }
-  if (string.indexOf('\n') !== -1) {
+  if (lineEnding.test(string)) {
+    this._sawReturn = /\r$/.test(string);
+
     // got one or more newlines; process into "line" events
-    var lines = string.split('\n');
+    var lines = string.split(lineEnding);
     // either '' or (concievably) the unfinished portion of the next line
     string = lines.pop();
     this._line_buffer = string;
     lines.forEach(function(line) {
-      this._onLine(line + '\n');
+      this._onLine(line);
     }, this);
   } else if (string) {
     // no newlines this time, save what we have for next time
@@ -586,7 +616,7 @@ Interface.prototype._ttyWrite = function(s, key) {
 
     switch (key.name) {
       case 'c':
-        if (this.listeners('SIGINT').length) {
+        if (EventEmitter.listenerCount(this, 'SIGINT') > 0) {
           this.emit('SIGINT');
         } else {
           // This readline instance is finished
@@ -633,6 +663,12 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._moveCursor(+1);
         break;
 
+      case 'l': // clear the whole screen
+        exports.cursorTo(this.output, 0, 0);
+        exports.clearScreenDown(this.output);
+        this._refreshLine();
+        break;
+
       case 'n': // next history item
         this._historyNext();
         break;
@@ -643,7 +679,7 @@ Interface.prototype._ttyWrite = function(s, key) {
 
       case 'z':
         if (process.platform == 'win32') break;
-        if (this.listeners('SIGTSTP').length) {
+        if (EventEmitter.listenerCount(this, 'SIGTSTP') > 0) {
           this.emit('SIGTSTP');
         } else {
           process.once('SIGCONT', (function(self) {
@@ -714,11 +750,23 @@ Interface.prototype._ttyWrite = function(s, key) {
   } else {
     /* No modifier keys used */
 
+    // \r bookkeeping is only relevant if a \n comes right after.
+    if (this._sawReturn && key.name !== 'enter')
+      this._sawReturn = false;
+
     switch (key.name) {
-      case 'enter':
+      case 'return':  // carriage return, i.e. \r
+        this._sawReturn = true;
         this._line();
         break;
 
+      case 'enter':
+        if (this._sawReturn)
+          this._sawReturn = false;
+        else
+          this._line();
+        break;
+
       case 'backspace':
         this._deleteLeft();
         break;
@@ -782,12 +830,14 @@ exports.Interface = Interface;
  */
 
 function emitKeypressEvents(stream) {
-  if (stream._emitKeypress) return;
-  stream._emitKeypress = true;
+  if (stream._keypressDecoder) return;
+  var StringDecoder = require('string_decoder').StringDecoder; // lazy load
+  stream._keypressDecoder = new StringDecoder('utf8');
 
   function onData(b) {
-    if (stream.listeners('keypress').length > 0) {
-      emitKey(stream, b);
+    if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
+      var r = stream._keypressDecoder.write(b);
+      if (r) emitKey(stream, r);
     } else {
       // Nobody's watching anyway
       stream.removeListener('data', onData);
@@ -802,7 +852,7 @@ function emitKeypressEvents(stream) {
     }
   }
 
-  if (stream.listeners('keypress').length > 0) {
+  if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
     stream.on('data', onData);
   } else {
     stream.on('newListener', onNewListener);
@@ -864,8 +914,12 @@ function emitKey(stream, s) {
 
   key.sequence = s;
 
-  if (s === '\r' || s === '\n') {
-    // enter
+  if (s === '\r') {
+    // carriage return
+    key.name = 'return';
+
+  } else if (s === '\n') {
+    // enter, should have been called linefeed
     key.name = 'enter';
 
   } else if (s === '\t') {
